; REPLACE.LSP Copyright 1993 by Looking Glass Microproducts
(defun C:REPLACE (/ ERROR PUSHVARS POPVARS NOTRANS 
                    REPLACE_MAIN PARSE VALID REPLACE_BLOCK 
                    PROCESS_BLOCK PROCESS_LINE REPLACE ITEM)
   ; Error Handler
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (princ S)
      )
      (if FHAND (close FHAND))
      (command "_undo" "end")
      (prompt "\nUndoing...")
      (command "_undo" "1")
      (POPVARS))
   ;============== Set and Save System Variables
   (defun PUSHVARS (VLIST)
      (foreach PAIR VLIST
         (setq
            SYSVARS (cons
                       (cons
                          (strcase (car PAIR))
                          (getvar
                             (car PAIR)
                          )
                       )
                       SYSVARS
                    )
         )
         (if (cdr PAIR) (setvar (car PAIR) (cdr PAIR)))
      ))
   ;============== Restore System Variables
   (defun POPVARS ()
      (foreach PAIR SYSVARS (setvar (car PAIR) (cdr PAIR)))
      (setq
         *error* OLD-ERROR
      )
      (setq SYSVARS nil)
      (princ))
   ;============= Disallow transparent invocation of routine.
   (defun NOTRANS ()
      (cond
         ((zerop (logand (getvar "cmdactive") (+ 1 2 4 8))))
         ((alert
           "This command may not be invoked transparently."
          )
         )
      ))
   ;============== Parse one input line
   (defun PARSE (LBUF / I LEN C TOKEN TOKENS)
      (setq
         LBUF  (strcat LBUF " ")
         I     0
         LEN   (strlen LBUF)
         TOKEN ""
      )
      (while (< I LEN)
         (setq I (1+ I) C (substr LBUF I 1))
         (if (= C " ")
            (if (/= TOKEN "")
               (setq
                  TOKENS (cons TOKEN TOKENS)
                  TOKEN  ""
               )
            )
            (setq TOKEN (strcat TOKEN C))
         )
      )
      (reverse TOKENS))
   ;================ Valid block/layer name
   (defun VALID (NAME)
      (if (not
           (or
             (not NAME)
             (= "" NAME)
             (wcmatch
              NAME
              "*[] `#`@`.`?`*`~`[`,`'!%^&()+={}|`\\:;\"<>/]*"
             )
           )
          )
         NAME
      ))
   ;=============== Item from association list
   (defun ITEM (I ALIST) (cdr (assoc I ALIST)))
   ;=============== Replace associated item
   (defun REPLACE (I ALIST)
      (subst I (assoc (car I) ALIST) ALIST))
   ;=============== Replace one block
   (defun REPLACE_BLOCK (ENAME TO_BLOCK TO_LAYER 
                         SCALE_FACTOR / ENT)
      (setq
         ENT (entget ENAME)
         ENT (REPLACE TO_BLOCK ENT)
         ENT (REPLACE TO_LAYER ENT)
         ENT (REPLACE
                (cons 41 (* SCALE_FACTOR (ITEM 41 ENT)))
                ENT
             )
         ENT (REPLACE
                (cons 42 (* SCALE_FACTOR (ITEM 42 ENT)))
                ENT
             )
         ENT (REPLACE (cons 43 (abs (ITEM 41 ENT))) ENT)
      )
      (entmod ENT))
   ;================ Process one block name
   (defun PROCESS_BLOCK (FROM_BLOCK FROM_LAYER TO_BLOCK 
                         TO_LAYER SCALE_FACTOR / ENAME)
      (cond
         ((progn
             (setq
                FILTER (list
                          (cons 0 "INSERT")
                          (cons 2 FROM_BLOCK)
                          (cons
                             8
                             FROM_LAYER
                          )
                       )
                SS     (ssget "x" FILTER)
                NUM    (if SS (sslength SS) 0)
             )
             (setvar
                "modemacro" (strcat
                               (itoa NUM)
                               "-"
                               FROM_BLOCK
                               " ==> "
                               TO_BLOCK
                               " @ "
                               TO_LAYER
                            )
             )
             (zerop NUM)
          )
         )
         (t
            (if (not (tblsearch "block" TO_BLOCK))
               (progn
                  (command "_insert" TO_BLOCK)
                  (command)
               )
            )
            (setq I 0)
            (while (< I NUM)
               (setq
                  ENAME (ssname SS I)
                  I     (1+ I)
               )
               (REPLACE_BLOCK
                  ENAME
                  (cons 2 TO_BLOCK)
                  (cons 8 TO_LAYER)
                  SCALE_FACTOR
               )
            )
            (setq SS nil)
         )
      ))
   ;================ Process one input line
   (defun PROCESS_LINE (LBUF LINENUM / LINE LINES 
                        FROM_BLOCK FROM_LAYER TO_BLOCK 
                        TO_LAYER SF SCALE_FACTOR LNUM)
      (setq 
           LNUM (itoa LINENUM)
           LINE (PARSE LBUF)
      )
      (cond
         ((/= 5 (length LINE))
            (alert
               (strcat
                  "ERROR:  "
                  "Incorrect number of arguments on line "
                  LNUM  ".\n\n" LBUF
               )
            )
         )
         ((progn
             (setq
                FROM_BLOCK (nth 0 LINE)
                FROM_LAYER (nth 1 LINE)
                TO_LAYER   (nth 2 LINE)
                TO_BLOCK   (nth 3 LINE)
                SF         (nth 4 LINE)
             )
             (not (VALID TO_BLOCK))
          )
            (alert
               (strcat
                  "ERROR:  Invalid Block Name "
                  TO_BLOCK
                  " on Line "
                  LNUM
                  ".\n\n"
                  LBUF
               )
            )
         )
         ((not
             (or
                (tblsearch "block" TO_BLOCK)
                (findfile
                   (strcat TO_BLOCK ".dwg")
                )
             )
          )
            (alert
               (strcat
                  "ERROR:  Unresolved Block "
                  TO_BLOCK
                  " on Line "
                  LNUM
                  ".\n\n"
                  LBUF
               )
            )
         )
         ((not (VALID TO_LAYER))
            (alert
               (strcat
                  "ERROR:  Invalid Layer Name "
                  TO_LAYER
                  " on Line "
                  LNUM
                  ".\n\n"
                  LBUF
               )
            )
         )
         ((or
             (not (setq SCALE_FACTOR (distof SF 2)))
             (<=
                SCALE_FACTOR
                0
             )
          )
            (alert
               (strcat
                  "ERROR:  Invalid Scale Factor "
                  SF
                  " on Line "
                  LNUM
                  ".\n\n"
                  LBUF
               )
            )
         )
         (t
            (list
               FROM_BLOCK
               FROM_LAYER
               TO_BLOCK
               TO_LAYER
               SCALE_FACTOR
            )
         )
      ))
   ;================= Replace main routine
   (defun REPLACE_MAIN (/ FNAME FHAND LBUF LINE LINES)
      (cond
         ((null
             (setq
                FNAME (getfiled
                         "Select Replacement File"
                         (strcat
                            (getvar "dwgname")
                            ".asc"
                         )
                         "asc"
                         6
                      )
             )
          )
         )
         ((null (setq FHAND (open FNAME "r")))
            (alert
               (strcat FNAME ":  Can't open file.")
            )
         )
         ((progn
             (setq LINENUM 0)
             (setvar
                "modemacro" (strcat "Reading " FNAME)
             )
             (while (and
                       (setq LBUF (read-line FHAND))
                       (setq
                          LINENUM (1+ LINENUM)
                       )
                       (setq
                          LINE (PROCESS_LINE LBUF LINENUM)
                       )
                    )
                (setq LINES (cons LINE LINES))
             )
             (close FHAND)
             (setq FHAND nil)
             LBUF
          )
         )
         (t
            (foreach LINE (reverse LINES)
               (eval (cons 'PROCESS_BLOCK LINE))
            )
         )
      )
   )
   ;============= Body of Replace Command 
   (if (NOTRANS)
      (progn
         (setq OLD-ERROR *error* *error* ERROR)
         (PUSHVARS
            '(("cmdecho" . 0) ("modemacro"))
         )
         (command "_undo" "group")
         (REPLACE_MAIN)
         (command "_undo" "end")
         (POPVARS)
      )
      (princ)
   )
)
(princ
   (strcat
      "  REPLACE.LSP (Copyright 1992 by "
      "Looking Glass Microproducts) loaded."
   )
)
(princ)
