;***************************************************************************
;*****                                                                 *****
;***                        TIME-DWG.LSP                                 ***
:*                                                                         *
;*                       Written by Doug Mann                              *
;*                                                                         *
;*             This AutoLISP routine can be used to time and               *
;*             report to a file, time spent in the drawing                 * 
;*             editor on a given file. The program reports to              *
;*             a file of the same name as the drawing but with             *
;*             a ".LOG" extension. The starting function is                *
;*             "(ask-first)". The user can type "PAUSE" at any             *
;*             time and suspend the timer. NOTE: A password is             *
;*             required to reactivate the timer and continue               *
;*             the editing session. An individual password or              *
;*             words may be entered on line 170 by removing "PASS"         *
;*             from this line and adding your own (capital letters!)       *
;*             (example: (initget 1 "PASSWORD1 PASSWORD2") ).              *
;***                                                                     ***
;*****                                                                 *****
;***************************************************************************

(defun clean-err (s)
  (if (/= s "Function cancelled")
    (princ (strcat "\nError: " s)))
  (if timers_cancelled
    (progn
      (print) (print)
      (prompt "\n *ALERT* Actions have caused Timer to ")
      (setq c:open nil c:new nil c:quit nil c:end nil
            c:pause nil c:stop nil start-timer nil
            open_log nil stop-timer nil ask-first nil
            running-time nil re-start nil timers_on nil
            S nil *error* olderr timers_cancelled nil)
      (setvar "cmdecho" 0)
      (command "redefine" "quit"
               "redefine" "open"
               "redefine" "new"
               "redefine" "end")
      (setvar "cmdecho" 1)
      (setvar "modemacro" mode)
      (if log_ (close log_))))
  (princ)
)

  (setq olderr  *error*
        *error* clean-err
        mode (getvar "modemacro"))

;************************************************************************

(defun start-timer (/ olderr)

  (setvar "modemacro"                           ;**********************
    (strcat                                     ; THIS SECTION MAY BE *
      "*TIMER* "                                ; OMITTED IF USER SO  *
      "Layer: $(substr,$(getvar,clayer),1,8)"   ; DESIRES, HOWEVER IT *
      "$(if,$(getvar,orthomode), Ortho)"        ; IS A NICE REMINDER. *
      "$(if,$(getvar,snapmode), Snap)"))        ; WINDOWS USERS PROB- *
                                                ; ABLY WON'T WANT IT. *
                                                ;**********************   
  (setq st_time (getvar "date")
        st_time (* 86400.0 (- st_time (fix st_time)))
        st_hour (fix (/ st_time 3600.0))
        st_min  (fix (/ (- st_time (* st_hour 3600.0)) 60.0))
        st_sec  (fix (- (- st_time (* st_hour 3600.0)) (* st_min 60.0)))
        timers_on t)
  (print) (print)
  (if re-start
    (princ "\nResuming timer at:  ")
    (progn
      (princ "\nType \"PAUSE\" to suspend or \"STOP\" to stop timer.")
      (princ "\nStarting time is:  ")))
  (setq re-start nil)
  (if (> st_hour 12)
    (setq st_hour (- st_hour 12)))
  (cond
    ((and (> st_min 10) (> st_sec 10))
      (princ (setq st-time (strcat (rtos st_hour 2 0) ":"
                           (rtos st_min 2 0) ":" (rtos st_sec 2 0)))))
    ((and (< st_min 10) (> st_sec 10))
      (princ (setq st-time (strcat (rtos st_hour 2 0) ":" "0"
                           (rtos st_min 2 0) ":" (rtos st_sec 2 0)))))
    ((and (> st_min 10) (< st_sec 10))
      (princ (setq st-time (strcat (rtos st_hour 2 0) ":"
                           (rtos st_min 2 0) ":" "0" (rtos st_sec 2 0)))))
    ((and (< st_min 10) (< st_sec 10))
      (princ (setq st-time (strcat (rtos st_hour 2 0) ":" "0"
                           (rtos st_min 2 0) ":" "0" (rtos st_sec 2 0))))))
  (setq dwg_ (strcat (setq dwgnam_ (strcase (getvar "dwgname"))) ".LOG")
        lgth_ (strlen dwgnam_))
  (while (and (/= dir_ "\\") (/= lgth_ 1))
    (setq lgth_ (1- lgth_))
    (setq dir_ (substr dwgnam_ lgth_ 1)))
  (if (= lgth_ 1) ()
    (setq dwgnam_ (substr dwgnam_ (1+ lgth_))))
  (setq dir_ nil)
  (princ)
)

;************************************************************************

(defun c:OPEN ()
  (setq timers_cancelled t)
  (print) (print)
  (initget "Yes No")
  (setq answ_ (getkword "\nReally want to discard all changes to drawing? <N>: "))
  (if (= answ_ "Yes")
    (progn
      (print) (print)
      (open_log)
      (setvar "cmdecho" 0)
      (command ".open" "y" (getfiled "Open Drawing" "" "dwg" 0))
      (setvar "cmdecho" 1)
      (setq re-start "go")          ;if user cancels out
      (setq timers_cancelled nil)   ;of dialog box
      (start-timer)))       
  (princ)
)

(defun c:NEW ()
  (setq timers_cancelled t)
  (print) (print)
  (initget "Yes No")
  (setq answ_ (getkword "\nReally want to discard all changes to drawing? <N>: "))
  (if (= answ_ "Yes")
    (progn
      (print)
      (open_log)
      (setvar "cmdecho" 0)
      (setq dn (getstring 1 "\nEnter new drawing name <RETURN for none>: "))
      (command ".new" "y" dn)
      (setvar "cmdecho" 1)
      (setq re-start "go")          ;if user cancels out
      (setq timers_cancelled nil)   ;of dialog box
      (start-timer)))
  (princ)
)

(defun c:QUIT ()
  (setq timers_cancelled t)
  (print) (print)
  (initget "Yes No")
  (setq answ_ (getkword "\nReally want to discard all changes to drawing? <N>: "))
  (if (= answ_ "Yes")
    (progn
      (print)
      (open_log)
      (setvar "cmdecho" 0)
      (command "delay" "2000")
      (setvar "cmdecho" 1) (print)
      (command ".quit" "y"))
    (setq timers_cancelled nil))
  (princ)
)

(defun c:END ()
  (open_log)
  (setvar "cmdecho" 0)
  (command "delay" "2000")
  (setvar "cmdecho" 1)
  (command ".end")
  (princ)
)

(defun c:PAUSE ()
  (setq re-start "go"
        timers_cancelled t)
  (initget 1 "PASS")
  (print)
  (open_log) (print)
  (alert "* TIMER IS SUSPENDED *") (print)         ;change "ALERT" to "PROMPT"
  (getkword "\nEnter \"PASSWORD\" to continue: ")  ;below AutoCAD-Release 12
  (setq timers_cancelled nil)
  (start-timer)
)

(defun c:STOP ()
  (print)
  (open_log)
  (setq c:open nil c:new nil c:quit nil c:end nil
        c:pause nil c:stop nil start-timer nil
        open_log nil stop-timer nil ask-first nil
        running-time nil re-start nil timers_on nil)
  (setvar "cmdecho" 0)
  (command "redefine" "quit"
           "redefine" "open"
           "redefine" "new"
           "redefine" "end")
  (setvar "cmdecho" 1)
  (setvar "modemacro" mode)
  (setq *error* olderr)
  (princ)
)

(defun c:START ()
  (if timers_on       ;don't restart if timers already on.
    (progn
      (print) (print)
      (prompt "\nTimer is already on!") (princ))
    (progn
      (setvar "cmdecho" 0)
      (command "undefine" "quit"
               "undefine" "open"
               "undefine" "new"
               "undefine" "end")
      (setvar "cmdecho" 1)
      (load "/lsp/time-dwg")  ;reload all of the functions again.
      (setq re-start nil)
      (start-timer)))
)

;************************************************************************


(defun open_log (/ file_ file_len drive_ line qty lgth_ dir_)
  (setq page "1")
  (if (null file_) (setq file_ (findfile dwg_)))
  (if file_
    (progn
      (setq its_1st_time nil
            file_len (strlen file_))
      (while (/= drive_ ":")
        (setq file_len (1- file_len))
        (setq drive_ (substr file_ file_len 1)))
      (setq file_ (substr file_ (1- file_len))
            drive_ nil
            log_ (open file_ "r")
            line "value"
            qty 0)
      (while (/= line nil)
        (if (= (substr line 1 1) (chr 12))
          (setq page (itoa (1+ (atoi page)))))  ;increment page number
        (setq qty (1+ qty)
              line (read-line log_)))
      (close log_)
      (setq  log_ (open file_ "r"))
      (repeat (- qty 2) (setq line (read-line log_)))
      (setq line (read-line log_)
            oldtime (substr line 47 9))
      (while (= (substr oldtime 1 1) " ")
        (setq oldtime (substr oldtime 2)))
      (close log_)
      (setq  log_ (open file_ "a"))
      (write-char 13 log_)
     (setq page-end 62)         ;length of page for form feed
     (while (< page-end 6200)
       (if (= qty page-end)
         (progn
           (setq page (itoa (1+ (atoi page))))
           (write-char 12 log_)
           (header)))
      (setq page-end (+ page-end 62))))    ;increment length of page
    (progn
      (setq file_ (strcat (getvar "dwgprefix") dwg_))
      (setq file_len (strlen file_))
      (while (/= drive_ ":")
        (setq file_len (1- file_len)
              drive_ (substr file_ file_len 1)))
      (setq file_ (substr file_ (1- file_len))
            drive_ nil
            log_ (open file_ "w")
            its_1st_time t)
      (header)))
  (stop-timer)
)

;************************************************************************

(defun header ()
  (repeat 3 (write-char 13 log_))
  (if (> (atoi page) 1)
    (princ "              Continuation of drawing named:  " log_)
    (princ "Record of time spent in drawing editor for drawing named:  " log_))
  (princ dwgnam_ log_)
  (if (= (strlen page) 1)
    (setq page (strcat " " page)))
  (repeat 2 (write-line "" log_))
  (write-line
  "  DATE         TIME       TIME      ELAPSED     RUNNING     USER" log_)
  (princ (strcat
  "                IN         OUT        TIME       TOTAL      NAME     Page# "
  page) log_)
  (write-char 13 log_)
  (repeat 77 (write-char 95 log_))
  (repeat 2 (write-char 13 log_))
)

;************************************************************************

(defun stop-timer (/ el_hour el_min el_sec hour min_ sec end_hour end_min
                     end_sec mth_ day_ yr_ date_)
  (setq end_time (getvar "date")
        end_time (* 86400.0 (- end_time (fix end_time)))
        end_hour (fix (/ end_time 3600.0))
        end_min  (fix (/ (- end_time (* end_hour 3600.0)) 60.0))
        end_sec  (fix (- (- end_time (* end_hour 3600.0)) (* end_min 60.0)))
        date_ (rtos (fix (getvar "cdate")) 2 0)
        mth_   (substr date_ 5 2)
        day_   (substr date_ 7 2)
        yr_    (substr date_ 1 4)
        date_ (strcat mth_ "/" day_ "/" yr_))

; * Print ending time to the command line and a .LOG file.

  (princ date_ log_)
  (repeat 3 (write-char 32 log_))
  (if (= (strlen st-time) 7)
    (progn
      (write-char 32 log_)
      (princ st-time log_))
    (princ st-time log_))
  (repeat 3 (write-char 32 log_))
  (if (or (> end_hour 12) (< end_hour 10))
    (write-char 32 log_))
  (if re-start
    (princ "\nEnding time at pause:  ")
    (progn
      (print)
      (princ "\nEnding time at exit:  ")))
  (if (> end_hour 12)
    (setq end_hour (- end_hour 12)))
  (cond
    ((and (> end_min 10) (> end_sec 10))
      (princ (setq end-time (strcat (rtos end_hour 2 0) ":"
                           (rtos end_min 2 0) ":" (rtos end_sec 2 0)))))
    ((and (< end_min 10) (> end_sec 10))
      (princ (setq end-time (strcat (rtos end_hour 2 0) ":" "0"
                           (rtos end_min 2 0) ":" (rtos end_sec 2 0)))))
    ((and (> end_min 10) (< end_sec 10))
      (princ (setq end-time (strcat (rtos end_hour 2 0) ":"
                           (rtos end_min 2 0) ":" "0" (rtos end_sec 2 0)))))
    ((and (< end_min 10) (< end_sec 10))
      (princ (setq end-time (strcat (rtos end_hour 2 0) ":" "0"
                           (rtos end_min 2 0) ":" "0" (rtos end_sec 2 0))))))
  (princ end-time log_)

; * Conversion for elapsed time.

  (if (< end_hour st_hour)
    (setq el_hour (- (+ end_hour 12) st_hour))
    (if (> end_hour 12)
      (setq el_hour (- (- end_hour 12) st_hour))
      (setq el_hour (- end_hour st_hour))))
  (if (< end_sec st_sec)
    (progn
      (if (< end_min st_min) 
        (progn
          (setq el_hour (- el_hour 1))
          (setq el_min (- (- (+ end_min 60) st_min) 1)))
        (setq el_min (- (- end_min st_min) 1)))
      (setq el_sec (- (+ end_sec 60) st_sec)))
    (progn
      (if (< end_min st_min) 
        (progn
          (setq el_hour (- el_hour 1))
          (setq el_min (- (+ end_min 60) st_min)))
        (setq el_min (- end_min st_min)))
      (setq el_sec (- end_sec st_sec))))

; * Print elapsed time to the command line and a .LOG file.

  (princ "  Elapsed time since start or last pause:  ")
  (repeat 2 (write-char 32 log_))
  (if (> el_hour 0)
    (setq hour (itoa el_hour)))
  (if (> el_min 0)
    (if (and (< el_min 10) (> el_hour 0))
      (setq min_ (strcat "0" (itoa el_min)))
      (setq min_ (itoa el_min))))
  (if (and (> el_hour 0) (= el_min 0))
    (setq min_"00"))
  (if (< el_sec 10)
    (setq sec (strcat "0" (itoa el_sec)))
    (setq sec (itoa el_sec)))
  (cond
    ((null min_) (setq newtime (strcat ":" sec)))
    ((and min_ (null hour)) (setq newtime (strcat min_ ":" sec)))
    ((and min_ hour) (setq newtime (strcat hour ":" min_ ":" sec))))
  (setq lgth (strlen newtime))
  (while (< lgth 9)
    (write-char 32 log_)
    (setq lgth (1+ lgth)))
  (princ newtime)
  (princ newtime log_)
  (repeat 3 (write-char 32 log_))
  (if its_1st_time
    (progn
      (while (< (strlen newtime) 9)
        (setq newtime (strcat " " newtime)))
      (princ newtime log_))
    (progn
      (running-time)
      (princ tottime log_)))
  (repeat 5 (write-char 32 log_))
  (princ (getvar "loginname") log_)
  (close log_)
  (setq *error* olderr)
)

;************************************************************************

(defun running-time (/ oldsec newsec oldmin newmin oldhour
                       newhour totsec totmin tothour)
  (setq oldsec (atoi (substr oldtime (- (strlen oldtime) 1) 2)))
  (setq newsec (atoi (substr newtime (- (strlen newtime) 1) 2)))

  (cond
    ((= (strlen oldtime) 4)
      (setq oldmin  (atoi (substr oldtime 1 1))))
    ((= (strlen oldtime) 5)
      (setq oldmin (atoi (substr oldtime 1 2))))
    ((= (strlen oldtime) 7)
      (setq oldmin (atoi (substr oldtime 3 2)))
      (setq oldhour (atoi (substr oldtime 1 1))))
    ((= (strlen oldtime) 8)
      (setq oldmin (atoi (substr oldtime 4 2)))
      (setq oldhour (atoi (substr oldtime 1 2))))
    ((= (strlen oldtime) 9)
      (setq oldmin (atoi (substr oldtime 5 2)))
      (setq oldhour (atoi (substr oldtime 1 3)))))

  (cond
    ((= (strlen newtime) 4)
      (setq newmin  (atoi (substr newtime 1 1))))
    ((= (strlen newtime) 5)
      (setq newmin (atoi (substr newtime 1 2))))
    ((= (strlen newtime) 7)
      (setq newmin (atoi (substr newtime 3 2)))
      (setq newhour (atoi (substr newtime 1 1))))
    ((= (strlen newtime) 8)
      (setq newmin (atoi (substr newtime 4 2)))
      (setq newhour (atoi (substr newtime 1 2))))
    ((= (strlen newtime) 9)
      (setq newmin (atoi (substr newtime 5 2)))
      (setq newhour (atoi (substr newtime 1 3)))))

  (setq totsec (+ oldsec newsec))

  (cond
    ((and (null oldmin) newmin)
      (setq totmin newmin))
    ((and (null newmin) oldmin)
      (setq totmin oldmin))
    ((and oldmin newmin)
      (setq totmin (+ oldmin newmin)))
    ((and (null oldmin) (null newmin))
      (setq totmin nil)))

  (cond
    ((and (null oldhour) newhour)
      (setq tothour newhour))
    ((and (null newhour) oldhour)
      (setq tothour oldhour))
    ((and oldhour newhour)
      (setq tothour (+ oldhour newhour)))
    ((and (null oldhour) (null newhour))
      (setq tothour nil)))

  (if (> totsec 59)
    (if totmin
      (progn
        (setq totsec (- totsec 60))
        (setq totmin (+ totmin 1)))
      (progn
        (setq totsec (- totsec 60))
        (setq totmin 1))))
  (if totmin
    (if (> totmin 59)
      (if tothour
        (progn
          (setq totmin (- totmin 60))
          (setq tothour (+ tothour 1)))
        (progn
          (setq totmin (- totmin 60))
          (setq tothour 1)))))

  (if (< totsec 10)
    (setq totsec (strcat "0" (itoa totsec)))
    (setq totsec (itoa totsec)))
  (if totmin
    (if (and (< totmin 10) tothour)
      (setq totmin (strcat "0" (itoa totmin)))
      (setq totmin (itoa totmin))))
  (if tothour
    (setq tothour (itoa tothour)))

  (cond
    ((null totmin) (setq tottime (strcat ":" totsec)))
    ((and totmin (null tothour)) (setq tottime (strcat totmin ":" totsec)))
    ((and totmin tothour) (setq tottime (strcat tothour ":" totmin ":" totsec))))
  (while (< (strlen tottime) 9)
    (setq tottime (strcat " " tottime)))
  (princ)
)

;************************************************************************

(defun ask-first ()
  (print) (print)
  (initget "Yes No")
  (setq answer
    (getkword "\nDo you wish to time this editing session? <Y>es or No: "))
  (if (= answer "No")
   (progn
     (print) (print)
     (prompt "* Timer is \"NOT\" on!")
     (princ))
   (c:start))
)
(princ) ;end Time-dwg.lsp
